@@ -36,7 +36,7 @@
 #define N_(s) gettext_noop(s)
 #include "common/error.h"
 #include "common/strleftcmp.h"
-#include "common/intparse.h"
+#include "common/intutil.h"
 #include "common/common.h"
 #include "common/strbuf.h"
 #include "microdc.h"
@@ -70,12 +70,29 @@ static void cmd_who(char *cmd, char *args);
 static void cmd_tag(char *cmd, char *args);
 static void cmd_transfers(char *cmd, char *args);
 static void cmd_cancel(char *cmd, char *args);
-static void cmd_debug(char *cmd, char *args);
 static void cmd_listenaddr(char *cmd, char *args);
 static void cmd_listenport(char *cmd, char *args);
 static void cmd_search(char *cmd, char *args);
 static void cmd_results(char *cmd, char *args);
 static void cmd_unsearch(char *cmd, char *args);
+static void cmd_display(char *cmd, char *args);
+
+static char *command_completion_generator(const char *base, int state);
+static bool display_completion_generator(const char *base, int state, DCCompletionEntry *ce);
+static char *speed_completion_generator(const char *base, int state);
+
+static struct {
+    DCDisplayFlag flag;
+    const char *name;
+} display_flag_names[] =  {
+    { DC_DF_DEBUG, "debug" },
+    { DC_DF_JOIN_PART, "joinpart" },
+    { DC_DF_PUBLIC_CHAT, "publicchat" },
+    { DC_DF_SEARCH_RESULTS, "searchresults" },
+    { DC_DF_UPLOAD, "upload" },
+    { DC_DF_DOWNLOAD, "download" },
+    { 0, NULL },
+};
 
 struct DCCommand {
     char *name;
@@ -91,9 +108,9 @@ static struct DCCommand commands[] = {
     { "cancel",         cmd_cancel,         DC_CPL_SIMPLE, transfer_completion_generator },
     { "cd", 	    	cmd_cd,     	    DC_CPL_FILE, remote_dir_completion_generator },
     { "connect",    	cmd_connect, 	    0, NULL },
-    { "debug",          cmd_debug,          0, NULL },
     { "description", 	cmd_description,    0, NULL },
     { "disconnect", 	cmd_disconnect,     0, NULL },
+    { "display",        cmd_display,        DC_CPL_CUSTOM, display_completion_generator },
     { "downloaddir", 	cmd_downloaddir,    DC_CPL_FILE, local_dir_completion_generator },
     { "email",      	cmd_email,  	    0, NULL },
     { "exit", 	    	cmd_exit,   	    0, NULL },
@@ -207,7 +224,7 @@ get_completor(const char *buffer, int start, int end, DCCompletionFunctionType *
  * can get faster than this - but remember that we don't compare more
  * commands than we eventually return to readline.
  */
-char *
+static char *
 command_completion_generator(const char *base, int state)
 {
     static int completion_start;
@@ -349,8 +366,10 @@ cmd_sharedir(char *cmd, char *args)
     	if (share_dir == NULL) {
     	    screen_putf(_("No share directory set\n"));
 	} else {
+	    char *fmt1 = ngettext("byte", "bytes", my_share_size);
     	    screen_putf(_("Current share directory: %s\n"), share_dir);
-    	    screen_putf(_("Currently sharing %" PRId64 " bytes (%" PRId64 " MB).\n"), my_share_size, my_share_size/(1024*1024));
+    	    screen_putf(_("Currently sharing %" PRId64" %s (%" PRId64 " MB).\n"),
+    	        my_share_size, fmt1, my_share_size/(1024*1024));
 	}
 	return;
     }
@@ -810,6 +829,7 @@ cmd_ls(char *cmd, char *args)
     free(path);
 }
 
+/* XXX: move to download.c? */
 static bool
 append_download_dir(DCUserInfo *ui, DCFileList *node, char *base_path, uint64_t *bytes, uint32_t *files)
 {
@@ -885,22 +905,22 @@ cmd_get(char *cmd, char *args)
     free(path);
 
     if (append_download_dir(browse_user, node, browse_path, &bytes, &files)) {
-	char *fmt;
-
-	fmt = ngettext("Downloading %s (%" PRId64 " bytes in %d file)\n",
-		"Downloading %s (%" PRId64 " bytes in %d files)\n", files);
+        char *fmt1 = ngettext("byte", "bytes", bytes);
+        char *fmt2 = ngettext("file", "files", files);
     	path = filelist_get_path(node);
-	screen_putf(fmt, path, bytes, files);
+	screen_putf(_("Downloading %s (%" PRId64 " %s in %d %s)\n"), path, bytes, fmt1, files, fmt2);
 	free(path);
     } else {
+        /* XXX: hmm, shouldn't we return even if queue is non-empty? */
     	if (browse_user->download_queue->cur == 0) {
 	    screen_putf(_("No files to download (empty directories).\n"));
 	    return;
 	}
     }
-    if (!has_user_conn(browse_user, DC_DIR_RECEIVE) && browse_user->conn_count < DC_USER_MAX_CONN)
+
+    if (!has_user_conn(browse_user, DC_DIR_RECEIVE) && browse_user->conn_count < DC_USER_MAX_CONN) {
     	hub_connect_user(browse_user); /* Ignore errors */
-    else {
+    } else {
 	char *msg;
 
 	msg = ngettext("No free connections. Queued file for download.\n",
@@ -939,6 +959,7 @@ cmd_unqueue(char *cmd, char *args)
     DCUserInfo *user;
     char *t, *t2;
     int32_t sp, ep;
+    uint32_t c;
 
     if (args == NULL) {
     	screen_putf(_("Usage: %s USER RANGE\n"), cmd);
@@ -947,43 +968,46 @@ cmd_unqueue(char *cmd, char *args)
 
     t = strchr(args, ' ');
     if (t == NULL) {
-        sp = 1;
+        sp = 0;
         ep = -1;
     } else {
         *t = '\0';
         for (t++; isspace(*t); t++);
         t2 = strchr(t, '-');
         if (t2 == NULL) { /* No dash. */
-            if (!parse_uint32(t, &sp)) {
+            if (!parse_int32(t, &sp) || sp <= 0) {
                 screen_putf(_("Invalid range\n"));
                 return;
             }
+            sp--;
 	    ep = sp+1;
         } else if (t == t2) { /* Starts with dash. */
-            sp = 1;
+            sp = 0;
             t++;
-            if (!parse_uint32(t, &ep)) {
+            if (!parse_int32(t, &ep) || ep <= 0) {
                 screen_putf(_("Invalid range\n"));
                 return;
             }
         } else if (t2[1] == '\0') { /* Ends with dash. */
             *t2 = '\0';
-            if (!parse_uint32(t, &sp)) {
+            if (!parse_int32(t, &sp) || sp <= 0) {
                 screen_putf(_("Invalid range\n"));
                 return;
             }
+            sp--;
 	    ep = -1;
 	} else { /* Dash in middle */
             *t2 = '\0';
             t2++;
-            if (!parse_uint32(t, &sp)) {
+            if (!parse_int32(t, &sp) || sp <= 0) {
                 screen_putf(_("Invalid range\n"));
                 return;
             }
-            if (!parse_uint32(t2, &ep)) {
+            if (!parse_int32(t2, &ep) || ep <= 0) {
                 screen_putf(_("Invalid range\n"));
                 return;
             }
+            sp--;
         }
     }
 
@@ -992,19 +1016,14 @@ cmd_unqueue(char *cmd, char *args)
         screen_putf(_("%s: No such user on this hub\n"), args);
         return;
     }
-
-    if (sp == 0 || ep == 0 || sp > user->download_queue->cur || ep > user->download_queue->cur) {
+    if (ep == -1)
+	ep = user->download_queue->cur;
+    if (sp >= user->download_queue->cur || ep > user->download_queue->cur) {
         screen_putf(_("Invalid range\n"));
         return;
     }
-    sp--;
-    if (ep < 0)
-        ep = user->download_queue->cur;
-    else
-    	ep--;
-
-    for (; sp < ep; sp++)
-	free_queued_file(user->download_queue->buf[sp]);
+    for (c = sp; c < ep; c++)
+	free_queued_file(user->download_queue->buf[c]);
     ptrv_remove_range(user->download_queue, sp, ep);
 }
 
@@ -1106,13 +1125,15 @@ cmd_who(char *cmd, char *args)
 	if (ui == NULL) {
     	    screen_putf(_("%s: No such user on this hub\n"), args);
 	} else {
+	    char *fmt1 = ngettext("byte", "bytes", ui->share_size);
     	    screen_putf(_("Nick: %s\n"), ui->nick);
     	    screen_putf(_("Description: %s\n"), ui->description);
     	    screen_putf(_("Speed: %s\n"), ui->speed);
     	    screen_putf(_("Level: %d\n"), ui->level);
     	    screen_putf(_("E-mail: %s\n"), ui->email);
     	    screen_putf(_("Operator: %d\n"), ui->is_operator);
-    	    screen_putf(_("Share Size: %" PRId64 " bytes (%" PRId64 " MB)\n"), ui->share_size, ui->share_size/(1024*1024));
+    	    screen_putf(_("Share Size: %" PRId64 " %s (%" PRId64 " MB)\n"),
+    	        ui->share_size, fmt1, ui->share_size/(1024*1024));
 	}
     	return;
     }
@@ -1226,19 +1247,6 @@ cmd_cancel(char *cmd, char *args)
     user_conn_cancel(uc);
 }
 
-static void
-cmd_debug(char *cmd, char *args)
-{
-    if (args == NULL) {
-    	screen_putf(_("Current debug level: %d\n"), debug_level);
-	return;
-    }
-    if (!parse_uint32(args, &debug_level)) {
-    	screen_putf(_("%s: Invalid integer\n"), args);
-	return;
-    }
-}
-
 static void
 cmd_listenaddr(char *cmd, char *args)
 {
@@ -1295,6 +1303,10 @@ cmd_search(char *cmd, char *args)
     	screen_putf(_("Usage: %s STRING...\n"), cmd);
 	return;
     }
+    if (hub_state < DC_HUB_LOGGED_IN) {
+    	screen_putf(_("Not connected.\n"));
+	return;
+    }
     add_search_request(args); /* Ignore errors */
 }
 
@@ -1304,9 +1316,20 @@ cmd_results(char *cmd, char *args)
     uint32_t c;
 
     if (args == NULL) {
+        time_t now;
+    
+	if (time(&now) < 0) {
+            warn(_("Cannot get current time - %s\n"), errstr);
+            return;
+        }
+
     	for (c = 0; c < our_searches->cur; c++) {
     	    DCSearchRequest *sd = our_searches->buf[c];
-	    screen_putf(_("%d. Results: %d\n"), c+1, sd->responses->cur);
+    	    char *status;
+
+            status = sd->issue_time + SEARCH_TIME_THRESHOLD <= now
+                   ? _("Closed") : _("Open");
+	    screen_putf(_("%d. (%s) Results: %d\n"), c+1, status, sd->responses->cur);
 	}
     } else {
     	DCSearchRequest *sd;
@@ -1366,3 +1389,96 @@ update_prompt(void)
 	set_screen_prompt("%s> ", PACKAGE);
     }
 }
+
+static uint32_t
+find_display_flag_value(const char *name)
+{
+    uint32_t c;
+    
+    for (c = 0; display_flag_names[c].flag != 0; c++) {
+	if (strcmp(display_flag_names[c].name, name) == 0)
+	    return display_flag_names[c].flag;
+    }
+
+    return 0;
+}
+
+static void
+cmd_display(char *cmd, char *args)
+{
+    uint32_t add_values = 0;
+    uint32_t set_values = 0;
+    uint32_t del_values = 0;
+    uint32_t c;
+    char *arg;
+
+    if (args == NULL) {
+        screen_putf(_("Current display flags:\n"));
+        for (c = 0; display_flag_names[c].flag != 0; c++) {
+            if (display_flags & display_flag_names[c].flag) {
+                screen_putf("  %s\n", display_flag_names[c].name);
+            }
+        }
+        return;
+    }
+
+    for (c = 0; (arg = get_argument(args, c, 1)) != NULL; c++) {
+	uint32_t *values;
+	uint32_t value;
+
+	if (arg[0] == '+') {
+	    values = &add_values;
+	    arg++;
+	} else if (arg[0] == '-') {
+	    values = &del_values;
+	    arg++;
+	} else {
+	    values = &set_values;
+	}
+	if (strcmp(arg, "all") == 0) {
+	    value = ~0;
+	} else if (strcmp(arg, "default") == 0) {
+	    value = ~0;
+	} else {
+	    value = find_display_flag_value(arg);
+	}
+	if (value == 0) {
+	    screen_putf(_("No flag by the name %s, display flags not changed.\n"), arg);
+	    return;
+	}
+	*values |= value;
+    }
+
+    if (set_values != 0 && (add_values != 0 || del_values != 0))
+	screen_putf(_("Cannot set and add or delete flags at the same time.\n"));
+
+    if (set_values != 0)
+	display_flags = set_values;
+    display_flags |= add_values;
+    display_flags &= ~del_values;
+}
+
+static bool
+display_completion_generator(const char *base, int state, DCCompletionEntry *ce)
+{
+    char base_char;
+    static uint32_t c;
+
+    base_char = (base[0] == '+' || base[0] == '-' ? *base++ : '\0');
+    if (state == 0)
+	c = 0;
+
+    for (; display_flag_names[c].flag != 0; c++) {
+	if (strleftcmp(base, display_flag_names[c].name) == 0) {
+	    if (base_char == '+')
+		ce->input_format = "+%s";
+	    else if (base_char == '-')
+		ce->input_format = "-%s";
+	    ce->str = xstrdup(display_flag_names[c].name);
+	    c++;
+	    return true;
+	}
+    }
+
+    return false;
+}
