@@ -21,9 +21,41 @@
 #include <stdlib.h>
 #include <string.h>
 #include "hmap.h"
-#include "hmap_internal.h"
 #include "xalloc.h"
 
+#define DEFAULT_CAPACITY    11
+#define DEFAULT_LOAD_FACTOR 0.75F
+
+typedef struct _HMapEntry HMapEntry;
+typedef struct _HMapIteratorPriv HMapIteratorPriv;
+
+struct _HMapEntry {
+    void *key;
+    void *value;
+    HMapEntry *next;
+};
+
+struct _HMap {
+    HMapEntry **buckets;
+    uint32_t buckets_length;
+    uint32_t threshold;
+    float load_factor;
+    uint32_t size;
+
+    hash_fn_t hash;
+    comparison_fn_t compare;
+};
+
+struct _HMapIteratorPriv {
+    bool (*has_next)(HMapIterator *it);
+    void *(*next)(HMapIterator *it);
+
+    HMap *map;
+    uint32_t index;
+    HMapEntry *entry;
+    HMapEntry *previous_entry;
+};
+
 static uint32_t
 strhash(const char *str)
 {
