@@ -64,13 +64,9 @@ void
 ptrv_append(PtrV *pv, void *value)
 {
     if (pv->cur >= pv->max) {
-        if (pv->max == 0)
-            pv->max = 1;
-        else
-            pv->max *= 2;
+        pv->max = MAX(1, pv->max*2);
         pv->buf = xrealloc(pv->buf, pv->max * sizeof(void *));
     }
-
     pv->buf[pv->cur] = value;
     pv->cur++;
 }
@@ -81,10 +77,7 @@ void
 ptrv_prepend(PtrV *pv, void *value)
 {
     if (pv->cur >= pv->max) {
-        if (pv->max == 0)
-            pv->max = 1;
-        else
-            pv->max *= 2;
+        pv->max = MAX(1, pv->max * 2);
         pv->buf = xrealloc(pv->buf, pv->max * sizeof(void *));
     }
     memmove(pv->buf+1, pv->buf, pv->cur * sizeof(void *));
@@ -99,7 +92,7 @@ void
 ptrv_remove_range(PtrV *pv, uint32_t start, uint32_t end)
 {
     if (end < pv->cur)
-        memmove(pv->buf + start, pv->buf + end, (pv->cur - end) * sizeof(void *));
+        memmove(pv->buf+start, pv->buf+end, (pv->cur - end) * sizeof(void *));
 
     pv->cur -= end - start;
 }
@@ -110,7 +103,7 @@ void *
 ptrv_remove_first(PtrV *pv)
 {
     void *value;
-    
+
     value = pv->buf[0];
     pv->cur--;
     if (pv->cur > 0)
@@ -162,10 +155,7 @@ ptrv_insort(PtrV *pv, void *element, comparison_fn_t comparator)
 	cmp = comparator(element, pv->buf[c]);
 	if (cmp <= 0) {
 	    if (pv->cur >= pv->max) {
-        	if (pv->max == 0)
-        	    pv->max = 1;
-        	else
-        	    pv->max *= 2;
+	        pv->max = MAX(1, pv->max * 2);
         	pv->buf = xrealloc(pv->buf, pv->max * sizeof(void *));
 	    }
 	    memmove(pv->buf+c+1, pv->buf+c, sizeof(void *)*(pv->cur-c));
