@@ -126,7 +126,7 @@ escape_message(char *str)
 void
 dump_command(const char *header, const char *buf, int len)
 {
-    if (debug_level > 0) {
+    if (display_flags & DC_DF_DEBUG) {
         StrBuf *out;
     	uint32_t c;
 
@@ -137,7 +137,7 @@ dump_command(const char *header, const char *buf, int len)
 	    else
 		strbuf_appendf(out, "\\x%02x", buf[c] & 0xFF);
 	}
-	screen_putf("%s %s\n", header, out->buf);
+	flag_putf(DC_DF_DEBUG, "%s %s\n", header, out->buf);
 	strbuf_free(out);
     }
 }
