@@ -38,7 +38,7 @@
 #include "common/common.h"
 #include "common/error.h"
 #include "common/substrcmp.h"
-#include "common/intparse.h"
+#include "common/intutil.h"
 #include "common/strbuf.h"
 #include "common/strleftcmp.h"
 #include "microdc.h"
@@ -191,6 +191,8 @@ filelist_lookup(DCFileList *node, const char *filename)
     return NULL;
 }
 
+/* Return the path of the node relative to the share root directory.
+ */
 char *
 filelist_get_path(DCFileList *node)
 {
@@ -209,6 +211,20 @@ filelist_get_path(DCFileList *node)
     return strbuf_free_to_string(sb);
 }
 
+/* Return the physical path of the node, by prepending the share directory path.
+ */
+static char *
+filelist_get_real_path(DCFileList *node)
+{
+    char *p1;
+    char *p2;
+    
+    p1 = filelist_get_path(node);
+    p2 = catfiles(share_dir, p1);
+    free(p1);
+    return p2;
+}
+
 static int
 file_node_compare(const void *i1, const void *i2)
 {
@@ -579,20 +595,34 @@ check_filename(const char *inname)
     return true;
 }
 
+/* Find the physical path of a file that the remote end
+ * wants to download from us.
+ */
 char *
 resolve_upload_file(DCUserInfo *ui, const char *name)
 {
-    /* Ideally, we would check our MyList.DcLst and look up a
-     * DCFileList node. Then we would get the path of this file
-     * using filelist_get_path, and prepend share_dir to that.
+    DCFileList *node;
+
+    /* Note: 'name' will have been translated to local slashes
+     * by a call to translate_remote_to_local prior to calling
+     * this function.
      */
-    if (!check_filename(name))
-    	return NULL; /* Invalid filename - contains .. components */
+
+    /* Skip leading slashes, all but one */
+    if (name[0] != '/')
+	return NULL;
+    for (; name[1] == '/'; name++);
+
     if (strcmp(name, "/MyList.DcLst") == 0)
-        return catfiles(listing_dir, name+1);
-    if (share_dir != NULL)
-        return catfiles(share_dir, name);
-    return NULL; /* Empty share directory */
+        return catfiles(listing_dir, name);
+
+    if (our_filelist == NULL || share_dir == NULL)
+        return NULL;
+
+    node = filelist_lookup(our_filelist, name);
+    if (node == NULL)
+        return NULL;
+    return filelist_get_real_path(node);
 }
 
 char *
@@ -676,12 +706,15 @@ local_fs_completion_generator(const char *dir_part, const char *file_part, int s
 		}
 
 		if (S_ISDIR(st.st_mode)) {
+		    /*ce->input_char = '/';*/
+		    /*ce->display_char = '/';*/
+		    ce->display_format = "%s/";
 		    ce->input_char = '/';
-		    ce->display_char = '/';
 		} else if (S_ISLNK(st.st_mode)) {
 		    if (stat(full, &st) >= 0 && S_ISDIR(st.st_mode)) {
+			ce->input_single_full_format = "%s/";
 		    	ce->input_char = '\0';
-			ce->input_append_single_full = '/';
+			/*ce->input_append_single_full = '/';*/
 		    } else {
 	    	    	if (dirsonly) {
 			    free(full);
@@ -689,7 +722,8 @@ local_fs_completion_generator(const char *dir_part, const char *file_part, int s
 			}
 			ce->input_char = ' ';
 		    }
-		    ce->display_char = '@';
+		    /*ce->display_char = '@';*/
+		    ce->display_format = "%s@";
 		} else {
 	    	    if (dirsonly) {
 			free(full);
@@ -697,9 +731,10 @@ local_fs_completion_generator(const char *dir_part, const char *file_part, int s
 		    }
 		    ce->input_char = ' ';
 		    if (access(full, X_OK) >= 0) {
-		    	ce->display_char = '*';
-		    } else {
-		    	ce->display_char = '\0';
+			ce->display_format = "%s*";
+		    	/*ce->display_char = '*';*/
+		    /*} else {*/
+		    	/*ce->display_char = '\0';*/
 		    }
 		}
 		ce->str = xstrdup(de->d_name);
@@ -780,10 +815,10 @@ remote_fs_completion_generator(const char *dir_path, const char *file_part, int
     	    ce->str = xstrdup(node->name);
 	    if (node->type == DC_TYPE_DIR) {
 		ce->input_char = '/';
-		ce->display_char = '/';
+		/*ce->display_char = '/';*/
 	    } else {
 		ce->input_char = ' ';
-		ce->display_char = '\0';
+		/*ce->display_char = '\0';*/
 	    }
 	    return true;
 	}
