@@ -18,7 +18,6 @@
  */
 
 #include <config.h>
-#include <ctype.h>		/* C89 */
 #include <string.h>		/* C89 */
 #include <stdlib.h>		/* C89 */
 #include <sys/socket.h>		/* ? */
@@ -33,7 +32,7 @@
 #define _(s) gettext(s)
 #define N_(s) gettext_noop(s)
 #include "common/error.h"
-#include "common/intparse.h"
+#include "common/intutil.h"
 #include "common/strleftcmp.h"
 #include "microdc.h"
 
@@ -65,7 +64,8 @@ say_user_completion_generator(const char *base, int state, DCCompletionEntry *ce
     	DCUserInfo *ui = it.next(&it);
 	if (strleftcmp(base, ui->nick) == 0) {
 	    ce->str = xstrdup(ui->nick);
-	    ce->input_append_single = ':';
+	    ce->input_single_format = "%s:";
+	    /*ce->input_append_single = ':';*/
 	    return true;
 	}
     }
@@ -356,7 +356,7 @@ hub_handle_command(char *buf, uint32_t len)
 	    	return;
     	    hub_state = DC_HUB_LOGGED_IN;
 	} else {
-	    screen_putf(_("User %s logged in.\n"), buf+7);
+	    flag_putf(DC_DF_JOIN_PART, _("User %s logged in.\n"), buf+7);
 	    ui = user_info_new(buf+7);
             hmap_put(hub_users, ui->nick, ui);
 	    if (!hub_putf("$GetINFO %s|", buf+7))
@@ -444,8 +444,8 @@ hub_handle_command(char *buf, uint32_t len)
 	/* Maybe we should be expecting an "hub follow" message? */
 	/* hub_disconnect(); */
     }
-    else if (len >= 3 && buf[0] == '<') {
-        screen_putf(_("Public: %s\n"), buf);
+    else if (len >= 3 && (buf[0] == '<' || strncmp(buf, " * ", 3) == 0)) {
+        flag_putf(DC_DF_PUBLIC_CHAT, _("Public: %s\n"), buf);
     }
     else if (len >= 5 && strncmp(buf, "$To: ", 5) == 0) {
     	char *msg;
@@ -569,10 +569,13 @@ hub_handle_command(char *buf, uint32_t len)
     }
     else if (len >= 6 && strncmp(buf, "$Quit ", 6) == 0) {
     	DCUserInfo *ui;
-    	screen_putf("User %s quits.\n", buf+6);
+    	flag_putf(DC_DF_JOIN_PART, "User %s quits.\n", buf+6);
 	ui = hmap_remove(hub_users, buf+6);
 	if (ui == NULL) {
-	    screen_putf(_("Invalid $Quit message: Unknown user %s.\n"), buf+6);
+	    /* Some hubs print quit messages for users that never joined,
+	     * so print this as debug only.
+	     */
+	    flag_putf(DC_DF_DEBUG, _("Invalid $Quit message: Unknown user %s.\n"), buf+6);
 	    return;
 	}
 	user_info_free(ui);
