@@ -31,6 +31,7 @@
 #include <inttypes.h>		/* ? */
 #include <fcntl.h>		/* ? */
 #include <getopt.h>		/* Gnulib/POSIX */
+#include <locale.h>		/* ? */
 #include "full-write.h"		/* Gnulib */
 #include "xvasprintf.h"		/* Gnulib */
 #include "version-etc.h"	/* Gnulib */
@@ -86,6 +87,7 @@ char *listing_dir;
 char *my_password;
 bool is_active;
 uint64_t my_share_size = 0;
+uint32_t display_flags = ~(DC_DF_DEBUG); /* All flags except debug set */
 
 /* interactive state */
 DCFileList *browse_list = NULL;
@@ -101,7 +103,6 @@ fd_set write_fds;
 PtrV *delete_files = NULL;
 PtrV *delete_dirs = NULL;
 static PtrV *search_udpmsg_out;	/* pending outgoing search results */
-uint32_t debug_level = 0;
 
 const char version_etc_copyright[] = "Copyright (C) 2004-2005 Oskar Liljeblad";
 
@@ -561,8 +562,10 @@ set_share_dir(const char *dir)
     	return false;
     if (hub_state >= DC_HUB_LOGGED_IN && !send_my_info())
     	return false;
-    if (dir != NULL)
-	screen_putf(_("Sharing %" PRId64 " bytes (%" PRId64 " MB) totally\n"), my_share_size, my_share_size/(1024*1024));
+    if (dir != NULL) {
+        char *fmt1 = ngettext("byte", "bytes", my_share_size);
+	screen_putf(_("Sharing %" PRId64 " %s (%" PRId64 " MB) totally\n"), my_share_size, fmt1, my_share_size/(1024*1024));
+    }
     return true;
 }
 
@@ -802,6 +805,15 @@ main (int argc, char **argv)
     char *tmpdir;
     struct sigaction sigact;
 
+#ifdef ENABLE_NLS
+    if (setlocale(LC_ALL, "") == NULL)
+       warn(_("%s: Cannot set locale: %s\n"), argv[0], errstr);
+    if (bindtextdomain(PACKAGE, LOCALEDIR) == NULL)
+       warn(_("%s: Cannot bind message domain: %s\n"), argv[0], errstr);
+    if (textdomain(PACKAGE) == NULL)
+        warn(_("%s: Cannot set message domain: %s\n"), argv[0], errstr);
+#endif
+
     while (true) {
 	c = getopt_long(argc, argv, "", long_opts, NULL);
 	if (c == -1)
