@@ -36,6 +36,17 @@
 #define DC_CLIENT_UDP_PORT 412
 #define DC_USER_MAX_CONN 2
 
+#define SEARCH_TIME_THRESHOLD 60        /* Add no more results to searches after this many seconds elapsed */
+
+typedef enum {
+    DC_DF_DEBUG			= 1 << 0, /* Various debug messages */
+    DC_DF_JOIN_PART		= 1 << 1, /* Joins and Quits messages */
+    DC_DF_PUBLIC_CHAT		= 1 << 2, /* Public chat */
+    DC_DF_SEARCH_RESULTS	= 1 << 3, /* Incoming search results */
+    DC_DF_UPLOAD		= 1 << 4, /* "Upload complete" message (not errors) */
+    DC_DF_DOWNLOAD		= 1 << 5, /* "Download complete" message (not errors) */
+} DCDisplayFlag;
+
 typedef enum {
     DC_CPL_SIMPLE,
     DC_CPL_FILE,
@@ -164,10 +175,15 @@ struct _DCSearchRequest {
 
 struct _DCCompletionEntry {
     char *str;
-    char input_char; /* character to append to input line */
-    char display_char; /* character to display */
-    char input_append_single; /* append to line buffer when this entry is the only match */
-    char input_append_single_full; /* as above, but only when the full string was entered before */
+    char input_char;
+    char *input_format;
+    char *display_format;
+    char *input_single_format;
+    char *input_single_full_format;
+//  char input_char; /* character to append to input line */
+//  char display_char; /* character to display */
+//  char input_append_single; /* append to line buffer when this entry is the only match */
+//  char input_append_single_full; /* as above, but only when the full string was entered before */
 };
 
 struct _DCFileList {
@@ -200,7 +216,7 @@ struct _DCUserInfo {
     char *email;
     uint64_t share_size;
     DCActiveState active_state;
-    PtrV *download_queue;   // FIXME LList?
+    PtrV *download_queue;   // XXX LList?
     uint16_t slot_granted;
     uint32_t refcount;
     bool info_quered;
@@ -250,6 +266,7 @@ extern struct in_addr force_listen_addr;
 extern int hub_socket;
 extern bool running;
 extern HMap *pending_userinfo;
+extern uint32_t display_flags;
 
 extern uint16_t listen_port;
 extern char *my_tag;
@@ -270,7 +287,6 @@ extern PtrV *delete_files;  /* XXX: use LList? */
 extern PtrV *delete_dirs;   /* XXX: use LList? */
 extern int used_ul_slots;
 extern int used_dl_slots;
-extern uint32_t debug_level;
 
 extern DCFileList *browse_list; /* list of user we browse */
 extern DCUserInfo *browse_user; /* user we browse OR WISH to browse, NULL if possibly browsing ourself */
@@ -289,7 +305,6 @@ void hub_disconnect(void);
 void hub_now_writable(void);
 bool send_my_info(void);
 char *user_completion_generator(const char *base, int state);
-char *speed_completion_generator(const char *base, int state);
 extern struct sockaddr_in hub_addr;
 extern char *hub_name;
 bool say_user_completion_generator(const char *base, int state, DCCompletionEntry *ce);
@@ -297,7 +312,6 @@ char *user_or_myself_completion_generator(const char *base, int state);
 
 /* command.c */
 void command_execute(char *line);
-char *command_completion_generator(const char *base, int state);
 void *get_completor(const char *buffer, int start, int end, DCCompletionFunctionType *type);
 /*bool append_download_dir(DCUserInfo *ui, DCFileList *node, char *base_path, uint64_t *bytes, uint32_t *files) XXX move transfer.c? */
 void update_prompt(void);
@@ -305,6 +319,7 @@ void update_prompt(void);
 /* screen.c */
 void screen_putf(const char *format, ...) __attribute__ ((format (printf, 1, 2)));
 void screen_vputf(const char *format, va_list args) __attribute__ ((format (printf, 1, 0)));
+void flag_putf(DCDisplayFlag flag, const char *format, ...);
 void screen_erase_and_new_line(void);
 void screen_finish(void);
 void screen_prepare(void);
@@ -312,6 +327,9 @@ void screen_read_input(void);
 void screen_get_size(int *rows, int *cols);
 char *get_argument(const char *p, int start, int count);
 void set_screen_prompt(const char *prompt, ...) __attribute__ ((format (printf, 1, 2)));
+//char *completion_redisplay_append(const char *input, bool only_match, void *data);
+//char *completion_redisplay_prepend(const char *input, bool only_match, void *data);
+//char *completion_redisplay_append_if_only_match(const char *input, bool only_match, void *data);
 
 /* huffman.c */
 uint8_t *huffman_decode(const uint8_t *data, uint32_t data_size, uint32_t *out_size);
