@@ -259,9 +259,9 @@ attempted_completion(const char *text, int start, int end)
     char *text_nq;
     char quoted;
 
-    quoted = ((char_is_quoted(rl_line_buffer, start) &&
-	    strchr(rl_completer_quote_characters, rl_line_buffer[start-1]) != NULL)
-	    ? rl_line_buffer[start-1] : 0);
+    /* Determine if current word is quoted, and dequote. */
+    quoted = char_is_quoted(rl_line_buffer, start)
+              && rl_line_buffer[start-1] == '"' ? '"' : '\0';
     text_nq = dequote_argument(text, quoted, INT_MAX);
    
     /* Do not attempt readline's default filename completion if our
@@ -270,8 +270,6 @@ attempted_completion(const char *text, int start, int end)
     rl_attempted_completion_over = 1;
 
     completor = get_completor(rl_line_buffer, start, end, &type);
-    //completor = path_completion_generator;
-
     if (type == DC_CPL_FILE) {
 	file_part = strrchr(text_nq, '/');
 	if (file_part == NULL) {
@@ -295,8 +293,9 @@ attempted_completion(const char *text, int start, int end)
 
     	completion_entries = ptrv_new();
 	for (state = 0; true; state++) {
-	    DCCompletionEntry ce = { NULL, ' ', '\0', '\0', '\0' };
-	    
+	    /*DCCompletionEntry ce = { NULL, ' ', '\0', '\0', '\0' };*/
+	    DCCompletionEntry ce = { NULL, ' ', "%s", "%s", NULL, NULL };
+
 	    if (type == DC_CPL_SIMPLE) {
 		DCCompletionFunction simple_completor = completor;
 		ce.str = simple_completor(file_part, state);
@@ -317,6 +316,7 @@ attempted_completion(const char *text, int start, int end)
     		DCCompletionEntry *oldce = completion_entries->buf[state-1];
 		int c1, c2, si;
 
+                /* Determine common leading characters for all matches. */
 	    	for (si = 0; (c1 = oldce->str[si]) && (c2 = ce.str[si]); si++) {
 		    if (c1 != c2)
 		    	break;
@@ -330,30 +330,38 @@ attempted_completion(const char *text, int start, int end)
 	    matches = NULL;
 	}
 	else if (state == 1) { /* single match */
-	    DCCompletionEntry *ce = completion_entries->buf[0];
-	    char *tmp = NULL;
+	    char *tmp;
+	    DCCompletionEntry *ce;
 
+            ce = completion_entries->buf[0];
 	    matches = xmalloc(sizeof(char *) * 2);
-	    if (ce->input_append_single) {
-		tmp = xasprintf("%s%c", ce->str, ce->input_append_single);
+
+	    if (ce->input_single_format != NULL) {
+		tmp = xasprintf(ce->input_single_format, ce->str);
 		free(ce->str);
 		ce->str = tmp;
-	    } else if (ce->input_append_single_full && strcmp(file_part, ce->str) == 0) {
+	    } else if (ce->input_single_full_format != NULL && strcmp(file_part, ce->str) == 0) {
 		free(ce->str);
-		ce->str = xasprintf("%s%c", file_part, ce->input_append_single_full);
+		ce->str = xasprintf(ce->input_single_full_format, file_part);
 	    }
 
 	    if (type == DC_CPL_FILE) {
 		tmp = catfiles(dir_part, ce->str);
+		free(ce->str);
+		ce->str = xasprintf(ce->input_format, tmp);
+		free(tmp);
 	    } else if (type == DC_CPL_SIMPLE || type == DC_CPL_CUSTOM) {
-		tmp = ce->str;
+		tmp = xasprintf(ce->input_format, ce->str);
+		free(ce->str);
+		ce->str = tmp;
 	    }
-	    matches[0] = quote_argument(tmp, SINGLE_MATCH, quoted);
-	    free(tmp);
 
+	    matches[0] = quote_argument(ce->str, SINGLE_MATCH, quoted);
 	    matches[1] = NULL;
+
 	    rl_completion_append_character = ce->input_char;
-	    free(ce);
+	    free(ce->str);
+    	    free(ce);
 	    ptrv_free(completion_entries);
 	}
     	else { /* multiple matches */
@@ -363,23 +371,28 @@ attempted_completion(const char *text, int start, int end)
     	    matches = xmalloc(sizeof(char *) * (state + 2));
 	    matches[0] = xstrndup(ce->str, lcd);
 	    if (type == DC_CPL_FILE) {
-		tmp = catfiles(dir_part, matches[0]);
+		char *tmp2;
+		tmp2 = catfiles(dir_part, matches[0]);
+		tmp = xasprintf(ce->input_format, tmp2);
+		free(tmp2);
 		free(matches[0]);
 	    }
 	    else if (type == DC_CPL_SIMPLE || type == DC_CPL_CUSTOM) {
-		tmp = matches[0];
+		tmp = xasprintf(ce->input_format, matches[0]);
+		free(matches[0]);
 	    }
 	    matches[0] = quote_argument(tmp, MULT_MATCH, quoted);
 	    free(tmp);
 
 	    for (c = 0; c < state; c++) {
 	    	ce = completion_entries->buf[c];
-		if (ce->display_char) {
-		    matches[c+1] = xasprintf("%s%c", ce->str, ce->display_char);
+		/*if (ce->display_format != NULL) {*/
+		    /*matches[c+1] = xasprintf("%s%c", ce->str, ce->display_char);*/
+		    matches[c+1] = xasprintf(ce->display_format, ce->str);
 		    free(ce->str);
-		} else {
+		/*} else {
 	    	    matches[c+1] = ce->str;
-		}
+		}*/
 	    }
 	    matches[state+1] = NULL;
 	    ptrv_foreach(completion_entries, free);
@@ -429,9 +442,6 @@ clear_rl()
         rl_expand_prompt(rl_prompt);
 }
 
-/* Print something on screen, va_list style.
- * __attribute__ is provided by mdc.h declaration.
- */
 static int
 real_screen_vputf(const char *format, va_list args)
 {
@@ -443,7 +453,7 @@ real_screen_vputf(const char *format, va_list args)
 }
 
 /* Print something on screen, printf format style.
- * __attribute__ is provided by mdc.h declaration.
+ * __attribute__ is provided by microdc.h declaration.
  */
 void
 screen_putf(const char *format, ...)
@@ -455,6 +465,18 @@ screen_putf(const char *format, ...)
     va_end(args);
 }
 
+void
+flag_putf(DCDisplayFlag flag, const char *format, ...)
+{
+    va_list args;
+
+    if (display_flags & flag) {
+	va_start(args, format);
+	screen_writer(format, args);
+	va_end(args);
+    }
+}
+
 /* This function is called by readline whenever the user has
  * entered a full line (usually by pressing enter).
  */
