@@ -32,18 +32,17 @@
 #define _(s) gettext(s)
 #define N_(s) gettext_noop(s)
 #include "common/strbuf.h"
-#include "common/intparse.h"
+#include "common/intutil.h"
 #include "common/common.h"
 #include "microdc.h"
 
 #define MAX_RESULTS_ACTIVE 10		/* Max number of search results to send to active users */
 #define MAX_RESULTS_PASSIVE 5		/* Max number of search results to send to passive users */
-#define SEARCH_TIME_THRESHOLD 60	/* Add no more results to searches after this many seconds elapsed */
 
 PtrV *our_searches;
 
 static char *extensions[] = { // NULL means match any extension
-    /* ANY */ NULL, 	
+    /* ANY */ NULL,
     /* AUDIO */ "mp3/mp2/wav/au/rm/mid/sm",
     /* COMPRESSED */ "zip/arj/rar/lzh/gz/z/arc/pak",
     /* DOCUMENTS */ "doc/txt/wri/pdf/ps/tex",
@@ -70,7 +69,7 @@ search_string_new(DCSearchString *sp, const char *p, int len)
     	sp->delta[(uint8_t) sp->str[c]] = len-c;
 }
 
-static void
+static bool
 parse_search_strings(char *str, DCSearchSelection *ss)
 {
     uint32_t c;
@@ -78,25 +77,28 @@ parse_search_strings(char *str, DCSearchSelection *ss)
     char *t2;
 
     ss->patterncount = 0;
-
     for (t1 = str; (t2 = strchr(t1, '$')) != NULL; t1 = t2+1) {
-    	if (t1 != str && t1[-1] != '$')
+    	if (t2 != str && t2[-1] != '$')
 	    ss->patterncount++;
     }
     if (*t1)
         ss->patterncount++;
+    if (ss->patterncount == 0)
+	return false;
 
     ss->patterns = xmalloc(sizeof(DCSearchString) * ss->patterncount);
 
     c = 0;
     for (t1 = str; (t2 = strchr(t1, '$')) != NULL; t1 = t2+1) {
-    	if (t1 != str && t1[-1] != '$') {
+    	if (t2 != str && t2[-1] != '$') {
     	    search_string_new(ss->patterns+c, t1, t2-t1);
 	    c++;
 	}
     }
     if (*t1)
        search_string_new(ss->patterns+c, t1, strlen(t1));
+
+    return true;
 }
 
 static bool
@@ -173,9 +175,7 @@ parse_search_selection(char *str, DCSearchSelection *data)
     }
     data->datatype = datatype[0]-'1';
 
-    parse_search_strings(str, data);
-
-    return true;
+    return parse_search_strings(str, data);
 }
 
 static bool
@@ -301,16 +301,14 @@ perform_inbound_search(DCSearchSelection *data, DCUserInfo *ui, struct sockaddr_
     maxresults = (ui == NULL ? MAX_RESULTS_ACTIVE : MAX_RESULTS_PASSIVE);
     curresults = filelist_search(our_filelist, data, maxresults, ui, addr);
 
-    if (debug_level > 0) {
-	if (curresults > 0) {
-	    if (ui != NULL) {
-    		screen_putf(_("Sent %d/%d search results to %s\n"), curresults, maxresults, ui->nick);
-	    } else {
-    		screen_putf(_("Sent %d/%d search results to %s\n"), curresults, maxresults, sockaddr_in_str(addr));
-	    }
-	} else {
-    	    screen_putf(_("No search results.\n"));
-	}
+    if (curresults > 0) {
+        if (ui != NULL) {
+            flag_putf(DC_DF_DEBUG, _("Sent %d/%d search results to %s\n"), curresults, maxresults, ui->nick);
+        } else {
+            flag_putf(DC_DF_DEBUG, _("Sent %d/%d search results to %s\n"), curresults, maxresults, sockaddr_in_str(addr));
+        }
+    } else {
+        flag_putf(DC_DF_DEBUG, _("No search results.\n"));
     }
 
     return false;
@@ -459,8 +457,9 @@ compare_search_selection(DCSearchSelection *s1, DCSearchSelection *s2)
     COMPARE_RETURN(s1->patterncount, s2->patterncount);
     for (c = 0; c < s1->patterncount; c++) {
     	COMPARE_RETURN(s1->patterns[c].len, s2->patterns[c].len);
-    	COMPARE_RETURN_FUNC(strncmp(s1->patterns[c].str, s2->patterns[c].str, s1->patterns[c].len));
+    	COMPARE_RETURN_FUNC(memcmp(s1->patterns[c].str, s2->patterns[c].str, s1->patterns[c].len));
     }
+    printf("no match!\n");
 
     return 0;
 }
@@ -481,7 +480,10 @@ add_search_request(char *args)
     sel.size_min = 0;
     sel.size_max = UINT64_MAX;
     sel.datatype = DC_SEARCH_ANY;
-    parse_search_strings(args, &sel);
+    if (!parse_search_strings(args, &sel)) {
+	warn(_("No pattern to match.\n"));
+	return false;
+    }
 
     for (c = 0; c < our_searches->cur; c++) {
     	sr = our_searches->buf[c];
@@ -555,7 +557,7 @@ handle_search_result(char *buf, uint32_t len)
 	    fmt = ngettext("Added search result to search %d (now %d result).\n",
 			   "Added search result to search %d (now %d results).\n",
 			   sd->responses->cur);
-	    screen_putf(fmt, c+1, sd->responses->cur);
+	    flag_putf(DC_DF_SEARCH_RESULTS, fmt, c+1, sd->responses->cur);
 	    sr->refcount++;
 	}
     }
