@@ -25,7 +25,6 @@
 #include <sys/stat.h>		/* ? */
 #include <sys/wait.h>		/* ? */
 #include <fcntl.h>		/* ? */
-#include <ctype.h>		/* C89 */
 #include <signal.h>		/* ? */
 #include <sys/socket.h>		/* ? */
 #include <netinet/in.h>		/* ? */
@@ -41,7 +40,7 @@
 #define _(s) gettext(s)
 #define N_(s) gettext_noop(s)
 #include "common/error.h"
-#include "common/intparse.h"
+#include "common/intutil.h"
 #include "common/common.h"
 #include "microdc.h"
 
@@ -78,7 +77,7 @@ static fd_set user_read_fds;
 static fd_set user_write_fds;
 static IPC *ipc;
 
-/* NOTE: The code below assumes that main never disconnects a user
+/* NOTE: All the code below assumes that main never disconnects a user
  * (close on main_socket) on purpose. The user must do this first if
  * things happen gracefully.
  */
@@ -132,7 +131,6 @@ handle_main_ipc_error(IPC *ipc, IPCError error, int result, void *userdata)
     	warn_socket_error(result, false, _("main process"));
 	break;
     case IPC_ERROR_DATA:
-    	//XXX: assert not reached
     	screen_putf(_("Logical communication error with main process.\n"));
     	break;
     }
@@ -298,6 +296,9 @@ open_download_file(void)
 	warn(_("%s: Cannot open file for writing - %s\n"), localfile, errstr);
 	//free(localfile); localfile = NULL;
 	//free(file_name/*DL*/); file_name/*DL*/ = NULL;
+	/* FIXME: should really try to download file later instead... */
+        //if (!ipc_put(ipc, DC_MSG_TRANSFER_FAILED/*DL*/, IPC_INT32, file_flag, IPC_END))
+        //   return;
 	return download_next_file();
     }
 
@@ -306,12 +307,16 @@ open_download_file(void)
 	if (close(transfer_file/*DL*/) < 0)
 	    warn(_("%s: Cannot close file - %s\n"), file_name/*DL*/, errstr);
 	transfer_file/*DL*/ = -1;
-	return download_next_file();
+    } else {
+        if (!user_putf("$Send|"))
+	    return false; /* maybe free file_name here? */
+        warn(_("Starting download for file %s, %" PRIu64 " bytes\n"), file_name/*DL*/, file_size);
     }
 
-    if (!user_putf("$Send|"))
-	return false; /* maybe free file_name here? */
-    warn(_("Starting download for file %s, %" PRIu64 " bytes\n"), file_name/*DL*/, file_size);
+    /* Let the main process think we are starting the download even if
+     * the file is empty and we are not going to download it.
+     * XXX: The code in main.c should be rewritten to handle this better.
+     */
     if (!ipc_put(ipc, DC_MSG_TRANSFER_STARTING/*DL*/, IPC_INT64, file_size, IPC_STR, file_name/*DL*/, IPC_END))
 	return false;
     if (!ipc_get(ipc, DC_MSG_TRANSFER_STARTING/*DL*/, IPC_BOOL, &reply, IPC_END))
@@ -321,19 +326,22 @@ open_download_file(void)
     	return false;
     }
 
-    file_pos = 0;
-    transfer_pos = 0;
-    data_size/*DL*/ = file_size;
-    user_state = DC_USER_DATA_RECV;
-
-    /* Places to go from here:
+    if (file_size == 0) {
+        if (!ipc_put(ipc, DC_MSG_TRANSFER_FINISHED/*DL*/, IPC_INT32, file_flag, IPC_END))
+           return false;
+	return download_next_file();
+    }
+    /* Places to go from here (assuming file_size != 0):
      *   user_running becomes false
      *     User process termination. user_disconnect deals with used_*_slots.
      *   data received until file complete
      *     Will issue DC_MSG_TRANSFER_FINISHED prior to calling
      *     download_next_file.
      */
-
+    file_pos = 0;
+    transfer_pos = 0;
+    data_size/*DL*/ = file_size;
+    user_state = DC_USER_DATA_RECV;
     return true;
 }
 
@@ -351,17 +359,18 @@ open_upload_file(const char *str, uint64_t offset)
     	free(newname);
 	return false;
     }
-    free(newname);
     if (!ipc_get(ipc, DC_MSG_RESOLVE_FILE, IPC_STR, &uploadfile, IPC_END))
     	return false;
 
     if (uploadfile == NULL) {
-        screen_putf(_("Cannot resolve upload file - invalid filename or no sharedir\n"));
+        screen_putf(_("Cannot resolve upload file `%s' - invalid filename or no sharedir\n"), newname);
+	free(newname);
 	if (!user_putf("$Error File Not Available|"))
 	    return false;
 	user_running = false;
 	return false;
     }
+    free(newname);
 
     free(file_name/*UL*/);
     file_name/*UL*/ = uploadfile;
@@ -464,7 +473,7 @@ user_handle_command(char *buf, uint32_t len)
 	data_size/*DL*/ -= len;
     	send_user_status(file_pos);
     	if (file_pos == file_size) {
-	    screen_putf(_("%s: Download complete.\n"), base_name(file_name/*DL*/));
+	    flag_putf(DC_DF_DOWNLOAD, _("%s: Download complete.\n"), base_name(file_name/*DL*/));
 	    if (close(transfer_file/*DL*/) < 0)
 	    	warn(_("%s: Cannot close file - %s\n"), file_name/*DL*/, errstr);
             if (file_flag == DC_TF_NORMAL) {
@@ -794,7 +803,7 @@ user_now_writable(void)
   	send_user_status(file_pos - user_sendq->cur);
 
 	if (file_pos == file_size && user_sendq->cur == 0) {
-	    screen_putf(_("%s: Upload complete.\n"), base_name(file_name/*UL*/));
+	    flag_putf(DC_DF_UPLOAD, _("%s: Upload complete.\n"), base_name(file_name/*UL*/));
     	    FD_CLR(user_socket, &user_write_fds);
 	    if (close(transfer_file/*UL*/) < 0)
 	    	warn(_("%s: Cannot close file - %s\n"), file_name/*UL*/, errstr);
@@ -961,8 +970,6 @@ user_main(int main_sockets[2], struct sockaddr_in *addr, int sock)
 
 cleanup:
 
-    /* FIXME: proper quit here!? */
-
     ipc_free(ipc);
     free(localfile);
     free(file_name);
@@ -1078,7 +1085,7 @@ user_conn_status_to_string(DCUserConn *uc, time_t now)
 	status = xasprintf(
 	    uc->dir == DC_DIR_RECEIVE
 		? _("Downloading %3d%% (at %5d kb/s) %s")
-		: _("Uploading %3d%% (at %5d kb/s) %s"),
+		: _("Uploading   %3d%% (at %5d kb/s) %s"),
 	    percent, rate, file);
     } else {
 	status = xasprintf(_("Idle"));
